@@ -41,7 +41,7 @@ Explode.initPreferences = function(pageWidget, calledByPrefDialog, document) {
         widgets["SplineTolerance"].visible = false;
         widgets["SplineTolerance_Label"].visible = false;
         widgets["SplinesToLineSegments"].visible = false;
-        widgets["Indent"].destroy();
+        destr(widgets["Indent"]);
         pageWidget.findChild("GridLayout").addWidget(widgets["SplineSegments_Label"], 2, 0, 1, 2);
     }
     else {
@@ -80,7 +80,9 @@ Explode.explodeSelection = function(di, toolTitle) {
     options["ellipseSegments"] = RSettings.getIntValue("Explode/EllipseSegments", 32);
     options["splinesToLineSegments"] = RSettings.getBoolValue("Explode/SplinesToLineSegments", false);
     options["textToPolylines"] = RSettings.getBoolValue("Explode/TextToPolylines", true);
+    options["textSplineToLineOrArc"] = RSettings.getBoolValue("Explode/TextSplineToLineOrArc", true);
     options["multilineTextToSimpleText"] = RSettings.getBoolValue("Explode/MultilineTextToSimpleText", true);
+    options["circlesToPolylines"] = RSettings.getBoolValue("Explode/CirclesToPolylines", true);
 
     var op = new RAddObjectsOperation();
     if (!isNull(toolTitle)) {
@@ -99,22 +101,16 @@ Explode.explodeSelection = function(di, toolTitle) {
             debugger;
         }
 
-        // these entity types are not explodable:
-        if (isPointEntity(entity) ||
-            isLineEntity(entity) ||
-            isXLineEntity(entity) ||
-            isRayEntity(entity) ||
-            isImageEntity(entity) ||
-            isArcEntity(entity) ||
-            isViewportEntity(entity)) {
-            continue;
-        }
-
         //var newShapes = [];
         //var newEntities = [];
 
         var news = Explode.explodeEntity(entity, options);
 
+        if (isNull(news)) {
+            // do nothing:
+            continue;
+        }
+
         // add explosion result and delete original:
         if (news.length!==0) {
             // add entities based on shapes that resulted from the explosion:
@@ -139,7 +135,7 @@ Explode.explodeSelection = function(di, toolTitle) {
                     e = shapeToEntity(entity.getDocument(), newShape);
                     if (!isNull(e)) {
                         e.setSelected(true);
-                        e.copyAttributesFrom(entity.data());
+                        e.copyAttributesFrom(getPtr(entity));
                         if (!isNull(newShape.color)) {
                             e.setColor(new RColor(newShape.color));
                         }
@@ -184,7 +180,7 @@ Explode.explodeSelection = function(di, toolTitle) {
 };
 
 /**
- * \return Array of new shapes and entities.
+ * \return Array of new shapes and entities or undefined to do nothing.
  */
 Explode.explodeEntity = function(entity, options) {
     var ret = [];
@@ -193,6 +189,17 @@ Explode.explodeEntity = function(entity, options) {
         options = {};
     }
 
+    // these entity types are not explodable:
+    if (isPointEntity(entity) ||
+        isLineEntity(entity) ||
+        isXLineEntity(entity) ||
+        isRayEntity(entity) ||
+        isImageEntity(entity) ||
+        isArcEntity(entity) ||
+        isViewportEntity(entity)) {
+        return undefined;
+    }
+
     var k, e, n, d;
     var col, row;
     var painterPaths;
@@ -203,9 +210,11 @@ Explode.explodeEntity = function(entity, options) {
     var splineTolerance = options["splineTolerance"];
     var splineSegments = options["splineSegments"];
     var ellipseSegments = options["ellipseSegments"];
-    var splinesToLineSegments = options["splinesToLineSegments"]
-    var textToPolylines = options["textToPolylines"]
+    var splinesToLineSegments = options["splinesToLineSegments"];
+    var textToPolylines = options["textToPolylines"];
+    var textSplineToLineOrArc = options["textSplineToLineOrArc"];
     var multilineTextToSimpleText = options["multilineTextToSimpleText"];
+    var circlesToPolylines = options["circlesToPolylines"];
 
     if (isNull(splineTolerance)) {
         splineTolerance = RSettings.getDoubleValue("Explode/SplineTolerance", 0.01);
@@ -222,9 +231,15 @@ Explode.explodeEntity = function(entity, options) {
     if (isNull(textToPolylines)) {
         textToPolylines = RSettings.getBoolValue("Explode/TextToPolylines", true);
     }
+    if (isNull(textSplineToLineOrArc)) {
+        textSplineToLineOrArc = RSettings.getBoolValue("Explode/TextSplineToLineOrArc", true);
+    }
     if (isNull(multilineTextToSimpleText)) {
         multilineTextToSimpleText = RSettings.getBoolValue("Explode/MultilineTextToSimpleText", true);
     }
+    if (isNull(circlesToPolylines)) {
+        circlesToPolylines = RSettings.getBoolValue("Explode/CirclesToPolylines", true);
+    }
 
     // explode ellipse into polyline with arc segments:
     if (isEllipseEntity(entity)) {
@@ -236,33 +251,120 @@ Explode.explodeEntity = function(entity, options) {
             }
         }
         else {
-            continue;
+            return undefined;
         }
     }
 
     // explode circle into polyline with two arc segments:
     else if (isCircleEntity(entity)) {
-        var circle = entity.getData().castToShape();
-        polyline = new RPolyline();
-        polyline.appendShape(new RArc(circle.getCenter(), circle.getRadius(), 0.0, Math.PI, false));
-        polyline.appendShape(new RArc(circle.getCenter(), circle.getRadius(), Math.PI, 2*Math.PI, false));
-        ret.push(polyline);
+        if (circlesToPolylines) {
+            var circle = entity.getData().castToShape();
+            polyline = new RPolyline();
+            polyline.appendShape(new RArc(circle.getCenter(), circle.getRadius(), 0.0, Math.PI, false));
+            polyline.appendShape(new RArc(circle.getCenter(), circle.getRadius(), Math.PI, 2*Math.PI, false));
+            polyline.autoClose();
+            ret.push(polyline);
+        }
+        else {
+            return undefined;
+        }
     }
 
     // explode polyline into line and arc segments:
     else if (isPolylineEntity(entity)) {
         polyline = entity.getData().castToShape();
+
+        // explode polyline with segment widths:
         if (RPolyline.hasProxy() && polyline.hasWidths()) {
-            var pls = polyline.getOutline();
+
+            // list of polyline pairs (left / right):
+            var pls;
+            if (RSettings.getQtVersion() >= 0x060000) {
+                pls = [];
+                var plsLeft = polyline.getLeftOutline();
+                var plsRight = polyline.getRightOutline();
+                for (var i=0; i<plsLeft.length && i<plsRight.length; i++) {
+                    pls.push([plsLeft[i], plsRight[i]]);
+                }
+            }
+            else {
+                pls = polyline.getLeftRightOutline();
+            }
+
+            var first = true;
+            var lastSegment = undefined;
+            var prevLeftPl = undefined;
+            var prevRightPl = undefined;
+
             for (k=0; k<pls.length; k++) {
-                ret.push(pls[k]);
+                if (pls[k].length!==2) {
+                    qWarning("invalid result from RPolyline::getLeftRightOutline");
+                    continue;
+                }
+
+                var leftPl = pls[k][0];
+                var rightPl = pls[k][1];
+
+                if (leftPl.countSegments()===1 && rightPl.countSegments()===1) {
+                    if (first) {
+                        // add orthogonal segment at start of this sequence:
+                        //ret.push(pls[k].getSegmentAt(3));
+                        if (isNull(prevLeftPl) || isNull(prevRightPl)) {
+                            ret.push(new RLine(leftPl.getStartPoint(), rightPl.getStartPoint()));
+                        }
+                        else {
+                            var dirLeft = prevLeftPl.getDirection2() + Math.PI;
+                            var dirRight = prevRightPl.getDirection2() + Math.PI;
+                            //qDebug("dirLeft:", dirLeft);
+                            //qDebug("new dir:", leftPl.getDirection1());
+                            if (!RMath.fuzzyAngleCompare(dirLeft, leftPl.getDirection1(), 0.01) ||
+                                !RMath.fuzzyAngleCompare(dirRight, rightPl.getDirection1(), 0.01)) {
+                                ret.push(new RLine(leftPl.getStartPoint(), rightPl.getStartPoint()));
+                            }
+                        }
+                    }
+
+                    // add left segment:
+                    ret.push(leftPl);
+                    // add right segment:
+                    ret.push(rightPl);
+
+                    lastSegment = new RLine(leftPl.getEndPoint(), rightPl.getEndPoint());
+                    prevLeftPl = leftPl;
+                    prevRightPl = rightPl;
+                }
+                //else if (leftPl.countSegments()>1 || rightPl.countSegments()>1) {
+                else if (!leftPl.isEmpty()) {
+                    //ret.push(pls[k]);
+                    if (!rightPl.isEmpty()) {
+                        ret.push(new RLine(leftPl.getStartPoint(), rightPl.getStartPoint()));
+                    }
+
+                    ret.push(leftPl);
+                    if (!rightPl.isEmpty()) {
+                        ret.push(rightPl);
+                        ret.push(new RLine(leftPl.getEndPoint(), rightPl.getEndPoint()));
+                    }
+                    first = true;
+                    if (!isNull(lastSegment)) {
+                        ret.push(lastSegment);
+                        lastSegment = undefined;
+                        prevLeftPl = undefined;
+                        prevRightPl = undefined;
+                    }
+
+                    continue;
+                }
+            }
+            if (!isNull(lastSegment)) {
+                ret.push(lastSegment);
             }
         }
         else {
             var polySegments = polyline.getExploded();
             if (polySegments.length>0) {
                 for (k=0; k<polySegments.length; k++) {
-                    var shape = polySegments[k].data();
+                    var shape = getPtr(polySegments[k]);
 
                     // last shape might have zero length if polyline is closed geometrically and logically:
                     if (k===polySegments.length-1) {
@@ -308,7 +410,7 @@ Explode.explodeEntity = function(entity, options) {
             for (k=0; k<painterPaths.length; k++) {
                 shapes = painterPaths[k].getShapes();
                 for (n=0; n<shapes.length; n++) {
-                    shape = shapes[n].data();
+                    shape = getPtr(shapes[n]);
                     if (isSplineShape(shape)) {
                         shape = ShapeAlgorithms.splineToLineOrArc(shape, 0.01);
                     }
@@ -322,12 +424,16 @@ Explode.explodeEntity = function(entity, options) {
     else if (isDimensionEntity(entity)) {
         shapes = entity.getShapes();
         for (k=0; k<shapes.length; k++) {
-            ret.push(shapes[k].data());
+            if (shapes[k].isValid()) {
+                ret.push(getPtr(shapes[k]));
+            }
         }
         var textData = entity.getData().getTextData();
         e = new RTextEntity(entity.getDocument(), textData);
         e.setSelected(true);
-        e.copyAttributesFrom(entity.data());
+        e.copyAttributesFrom(getPtr(entity));
+        var textColor = document.getKnownVariable(RS.DIMCLRT, new RColor(RColor.ByBlock));
+        e.setColor(textColor);
         ret.push(e);
     }
 
@@ -335,16 +441,37 @@ Explode.explodeEntity = function(entity, options) {
     else if (isLeaderEntity(entity)) {
         shapes = entity.getShapes();
         for (k=0; k<shapes.length; k++) {
-            shape = shapes[k].data();
+            shape = getPtr(shapes[k]);
             ret.push(shape.clone());
         }
     }
 
+    // explode tolerances into lines and texts:
+    else if (isToleranceEntity(entity)) {
+        shapes = entity.getFrame();
+        for (k=0; k<shapes.length; k++) {
+            shape = shapes[k];
+            ret.push(shape.clone());
+        }
+
+        var textDatas = entity.getTextLabels();
+        for (k=0; k<textDatas.length; k++) {
+            var d = textDatas[k];
+            e = new RTextEntity(document, new RTextData(d))
+            e.setSelected(true);
+            e.copyAttributesFrom(getPtr(entity));
+            if (e.getColor()!==d.getColor()) {
+                e.setColor(d.getColor());
+            }
+            ret.push(e);
+        }
+    }
+
     // explode solids into polylines:
     else if (isSolidEntity(entity)) {
         shapes = entity.getShapes();
         for (k=0; k<shapes.length; k++) {
-            shape = shapes[k].data();
+            shape = getPtr(shapes[k]);
             if (shape.countVertices()===4) {
                 var v3 = shape.getVertexAt(3);
                 shape.setVertexAt(3, shape.getVertexAt(2));
@@ -358,12 +485,12 @@ Explode.explodeEntity = function(entity, options) {
     else if (isFaceEntity(entity)) {
         shapes = entity.getShapes();
         for (k=0; k<shapes.length; k++) {
-            shape = shapes[k].data();
+            shape = getPtr(shapes[k]);
             ret.push(shape.clone());
         }
     }
 
-    // explode text entities into lines, arcs and splines:
+    // explode text entities into text blocks or lines, arcs and splines:
     else if (isTextEntity(entity)) {
         // explode multi-block text into simple text entities:
         // each text block with the same format is converted into one individual simple text entity:
@@ -374,7 +501,7 @@ Explode.explodeEntity = function(entity, options) {
                 var d = textDatas[k];
                 e = new RTextEntity(document, new RTextData(d))
                 e.setSelected(true);
-                e.copyAttributesFrom(entity.data());
+                e.copyAttributesFrom(getPtr(entity));
                 if (e.getColor()!==d.getColor()) {
                     e.setColor(d.getColor());
                 }
@@ -412,7 +539,9 @@ Explode.explodeEntity = function(entity, options) {
                     shape = shapes[n];
                     if (isSplineShape(shape)) {
                         // spline to arc or line or spline:
-                        shape = ShapeAlgorithms.splineToLineOrArc(shape, splineTolerance);
+                        if (textSplineToLineOrArc) {
+                            shape = ShapeAlgorithms.splineToLineOrArc(shape, splineTolerance);
+                        }
 
                         if (textToPolylines) {
                             // spline to polyline with arcs:
@@ -470,7 +599,12 @@ Explode.explodeEntity = function(entity, options) {
         d.setParentId(RObject.INVALID_ID);
         e = new RTextEntity(document, d);
         e.setSelected(true);
-        e.copyAttributesFrom(entity.data());
+        if (isFunction(entity.data)) {
+            e.copyAttributesFrom(getPtr(entity));
+        }
+        else {
+            e.copyAttributesFrom(entity);
+        }
         ret.push(e);
     }
 
@@ -478,6 +612,7 @@ Explode.explodeEntity = function(entity, options) {
     else if (isBlockReferenceEntity(entity)) {
         var data = entity.getData();
         var pos = entity.getPosition();
+        var uniform = RMath.fuzzyCompare(entity.getScaleFactors().x, entity.getScaleFactors().y);
 
         // explode array into multiple block references:
         if (data.getColumnCount()>1 || data.getRowCount()>1) {
@@ -505,11 +640,21 @@ Explode.explodeEntity = function(entity, options) {
                     continue;
                 }
 
+                if (!isBlockReferenceEntity(subEntity)) {
+                    subEntity.scaleVisualProperties(data.getScaleFactors().x);
+                }
+
                 // ignore attribute definitions:
                 if (isAttributeDefinitionEntity(subEntity)) {
                     continue;
                 }
 
+                // ignore hatch in non-uniformly scaled block reference:
+                if (!uniform && isHatchEntity(subEntity)) {
+                    EAction.handleUserWarning(qsTr("Ignored hatch in block reference with non-uniform scale factors"));
+                    continue;
+                }
+
                 e = subEntity.clone();
                 //data.applyColumnRowOffsetTo(e, col, row);
                 // we need the old ID to update attribute parent IDs:
