@@ -47,7 +47,7 @@ function version() {
  * Prints command line usage information on stdout.
  */
 function usage() {
-    print("\nUsage: " + QCoreApplication.arguments()[0] + " [Options] [Files to open]\n"
+    print("\nUsage: " + RSettings.getOriginalArguments()[0] + " [Options] [Files to open]\n"
           + "\n"
           + "-allow-multiple-instances        Don't try to avoid multiple instances from running\n"
           + "                                 simultaneously.\n"
@@ -94,132 +94,6 @@ function usage() {
     print("\n");
 }
 
-/**
- * Open files given as arguments args
- *
- * \param createNew Creates a new document if no files are given
- * \param close Closes existing open MDI widgets
- */
-function openFiles(args, createNew, close) {
-    var appWin = RMainWindowQt.getMainWindow();
-    if (isNull(appWin)) {
-        // application is shutting down..
-        return;
-    }
-    var mdiArea = appWin.getMdiArea();
-    var mdiChildren = mdiArea.subWindowList();
-    var foundFile = false;
-    var filter = undefined;
-
-    for (var i = 0; i < args.length; ++i) {
-        // arguments with one parameter:
-        if (args[i] === "-locale" || args[i] === "-autostart"
-            || args[i] === "-app-id" || args[i] === "-ignore"
-            || args[i] === "-config") {
-            // skip 2 arguments
-            i++;
-            if (i>=args.length) {
-                break;
-            }
-            continue;
-        }
-
-        // argument with two parameters
-        if (args[i] === "-font-substitution" || args[i] === "-fs" || args[i] === "-ts") {
-            // skip 3 arguments
-            i+=2;
-            if (i>=args.length) {
-                break;
-            }
-            continue;
-        }
-
-        if (isNull(args[i])) {
-            continue;
-        }
-
-        // all arguments after -exec are script files or script arguments:
-        if (args[i] === "-exec") {
-            break;
-        }
-
-        if (args[i] === "-filter") {
-            if (++i>=args.length) {
-                break;
-            }
-            filter = args[i];
-            continue;
-        }
-
-
-        // skip other arguments without parameter:
-        if (args[i][0] === "-") {
-            continue;
-        }
-
-        foundFile = true;
-        var foundExisting = false;
-
-        var arg = args[i];
-        var isLocalFile = true;
-
-        if (isUrl(arg)) {
-            var url = new QUrl(arg);
-            if (url.isLocalFile()) {
-                // arg is now a path:
-                arg = url.toLocalFile();
-            }
-            else {
-                isLocalFile = false;
-            }
-        }
-
-        if (isLocalFile) {
-            // if the file is already open, activate that appropriate sub window instead
-            // of opening the file again:
-            var document = undefined;
-            var fileName = undefined;
-            var fileInfo = undefined;
-            var argFileInfo = undefined;
-            for (var k=0; k<mdiChildren.length; k++) {
-                document = mdiChildren[k].getDocument();
-                fileName = document.getFileName();
-                fileInfo = new QFileInfo(fileName);
-                argFileInfo = new QFileInfo(getAbsolutePathForArg(arg));
-
-                if (fileInfo.absoluteFilePath()===argFileInfo.absoluteFilePath()) {
-                    mdiArea.setActiveSubWindow(mdiChildren[k]);
-                    if (close) {
-                        mdiArea.closeActiveSubWindow();
-                    }
-                    else {
-                        foundExisting = true;
-                    }
-                    break;
-                }
-            }
-        }
-
-        // open the file if it is not already open:
-        if (!foundExisting) {
-            if (isLocalFile) {
-                NewFile.createMdiChild(getAbsolutePathForArg(arg), filter);
-            }
-            else {
-                NewFile.createMdiChild(arg, filter);
-            }
-        }
-    }
-
-    // create new document if no files were loaded:
-    if (!foundFile && createNew===true) {
-        var fileNewAction = RGuiAction.getByScriptFile("scripts/File/NewFile/NewFile.js");
-        if (!isNull(fileNewAction)) {
-            fileNewAction.slotTrigger();
-        }
-    }
-}
-
 /**
  * Executes the scripts that are given as arguments of -exec.
  * \return true if the application should quit after executing the
@@ -288,7 +162,7 @@ function setUpDragAndDrop(appWin) {
         event.acceptProposedAction();
     });
 
-    appWin.drop.connect(function(evt) {
+    appWin.drop.connect(function(event) {
         // workaround for Qt keyboard focus bug:
         var appWin = RMainWindowQt.getMainWindow();
         if (!isNull(appWin)) {
@@ -297,13 +171,15 @@ function setUpDragAndDrop(appWin) {
             appWin.setFocus(Qt.OtherFocusReason);
         }
 
-        var urls = getUrlsFromMimeData(evt.mimeData());
+        var urls = getUrlsFromMimeData(event.mimeData());
         var urlStrings = [];
         for (var i = 0; i < urls.length; ++i) {
             urlStrings.push(urls[i].toString());
         }
 
         openFiles(urlStrings, false);
+
+        event.acceptProposedAction();
     });
 }
 
@@ -320,7 +196,7 @@ function loadTranslations(addOns, splash) {
 
     // load C++ translations:
     var modules = ["qt", "assistant", "qt_help", "qcadcore", "qcadentity", "qcadgui"];
-    if (RSettings.isQt(5)) {
+    if (RSettings.isQt(5) || RSettings.isQt(6)) {
         modules.unshift("qtbase");
     }
 
@@ -336,7 +212,7 @@ function loadTranslations(addOns, splash) {
     //RSettings.loadTranslations("scripts_" + locale, [autoPath("scripts/ts")]);
 
     // load translations from arguments:
-    var args = QCoreApplication.arguments();
+    var args = RSettings.getOriginalArguments();
     for (i = 0; i < args.length; ++i) {
         if (args[i] === "-ts") {
             if (++i>=args.length) {
@@ -562,7 +438,7 @@ function main() {
     }
 
     // detect very first start of this installation:
-    var fiSettings = new QFileInfo(RSettings.getQSettings().fileName());
+    var fiSettings = new QFileInfo(RSettings.getFileName());
     var isFirstStart = !fiSettings.exists();
 
     var numPlugins = RPluginLoader.countPlugins();
@@ -571,8 +447,8 @@ function main() {
     // look up app name override:
     for (i=0; i<numPlugins; i++) {
         pluginInfo = RPluginLoader.getPluginInfo(i);
-        var n = pluginInfo.get("NameOverride");
-        if (!isNull(n)) {
+        var n = pluginInfo.get("NameOverride", "");
+        if (n.length>0) {
             qApp.applicationName = n;
         }
     }
@@ -729,7 +605,14 @@ function main() {
     //RDebug.stopTimer(0, "initializing add-ons");
 
     // auto load scripts in AutoLoad folders for global script engine:
-    var files = RAutoLoadEcma.getAutoLoadFiles();
+
+    var files;
+    if (RSettings.isQt(6)) {
+        files = RAutoLoadJs.getAutoLoadFiles();
+    }
+    else {
+        files = RAutoLoadEcma.getAutoLoadFiles();
+    }
     for (i=0; i<files.length; i++) {
         include(files[i]);
     }
@@ -767,7 +650,7 @@ function main() {
 
     if (!isNull(splash)) {
         splash.close();
-        splash.destroy();
+        destr(splash);
     }
 
     postInitAddOns(addOns);
@@ -844,7 +727,7 @@ function main() {
     // don't use RSettings below this point
 
     // and we're done:
-    appWin.destroy();
+    destr(appWin);
     qDebug("done");
 }
 
